@@ -1,5 +1,3 @@
-# All Rights Reserved.
-#
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
 #    a copy of the License at
@@ -12,21 +10,27 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from __future__ import annotations
 
 import os
 import tempfile
+import typing
+from typing import Any, Optional, Union  # noqa: H301
+
+if typing.TYPE_CHECKING:
+    import io
 
 from oslo_concurrency import processutils as putils
 from oslo_log import log as logging
 from oslo_serialization import jsonutils
 from oslo_utils import excutils
 from oslo_utils import fileutils
-from oslo_utils import netutils
 
 from os_brick import exception
 from os_brick.i18n import _
 from os_brick import initiator
 from os_brick.initiator.connectors import base
+from os_brick.initiator.connectors import base_rbd
 from os_brick.initiator import linuxrbd
 from os_brick.privileged import rbd as rbd_privsep
 from os_brick import utils
@@ -34,14 +38,15 @@ from os_brick import utils
 LOG = logging.getLogger(__name__)
 
 
-class RBDConnector(base.BaseLinuxConnector):
+class RBDConnector(base_rbd.RBDConnectorMixin, base.BaseLinuxConnector):
     """"Connector class to attach/detach RBD volumes."""
 
     def __init__(self, root_helper, driver=None, use_multipath=False,
                  device_scan_attempts=initiator.DEVICE_SCAN_ATTEMPTS_DEFAULT,
                  *args, **kwargs):
 
-        super(RBDConnector, self).__init__(root_helper, driver=driver,
+        super(RBDConnector, self).__init__(root_helper,
+                                           driver=driver,  # type: ignore
                                            device_scan_attempts=
                                            device_scan_attempts,
                                            *args, **kwargs)
@@ -52,29 +57,27 @@ class RBDConnector(base.BaseLinuxConnector):
         """The RBD connector properties."""
         return {'do_local_attach': kwargs.get('do_local_attach', False)}
 
-    def get_volume_paths(self, connection_properties):
+    def get_volume_paths(self,
+                         connection_properties: dict[str, Any]) -> list[str]:
         # TODO(e0ne): Implement this for local volume.
         return []
 
-    def get_search_path(self):
+    def get_search_path(self) -> None:
         # TODO(walter-boring): don't know where the connector
         # looks for RBD volumes.
         return None
 
-    def get_all_available_volumes(self, connection_properties=None):
+    def get_all_available_volumes(
+            self,
+            connection_properties: Optional[dict[str, Any]] = None) -> \
+            list[str]:
         # TODO(e0ne): Implement this for local volume.
         return []
 
     @staticmethod
-    def _sanitize_mon_hosts(hosts):
-        def _sanitize_host(host):
-            if netutils.is_valid_ipv6(host):
-                host = '[%s]' % host
-            return host
-        return list(map(_sanitize_host, hosts))
-
-    @staticmethod
-    def _check_or_get_keyring_contents(keyring, cluster_name, user):
+    def _check_or_get_keyring_contents(keyring: Optional[str],
+                                       cluster_name: str,
+                                       user: str) -> str:
         try:
             if keyring is None:
                 if user:
@@ -90,8 +93,12 @@ class RBDConnector(base.BaseLinuxConnector):
             raise exception.BrickException(msg=msg)
 
     @classmethod
-    def _create_ceph_conf(cls, monitor_ips, monitor_ports,
-                          cluster_name, user, keyring):
+    def _create_ceph_conf(cls,
+                          monitor_ips: list[str],
+                          monitor_ports: list[str],
+                          cluster_name: str,
+                          user: str,
+                          keyring) -> str:
         monitors = ["%s:%s" % (ip, port) for ip, port in
                     zip(cls._sanitize_mon_hosts(monitor_ips), monitor_ports)]
         mon_hosts = "mon_host = %s" % (','.join(monitors))
@@ -113,7 +120,8 @@ class RBDConnector(base.BaseLinuxConnector):
             msg = (_("Failed to write data to %s.") % (ceph_conf_path))
             raise exception.BrickException(msg=msg)
 
-    def _get_rbd_handle(self, connection_properties):
+    def _get_rbd_handle(self, connection_properties: dict[str, Any]) -> \
+            linuxrbd.RBDVolumeIOWrapper:
         try:
             user = connection_properties['auth_username']
             pool, volume = connection_properties['name'].split('/')
@@ -143,32 +151,8 @@ class RBDConnector(base.BaseLinuxConnector):
 
         return rbd_handle
 
-    @classmethod
-    def _get_rbd_args(cls, connection_properties, conf=None):
-        try:
-            user = connection_properties['auth_username']
-            monitor_ips = connection_properties.get('hosts')
-            monitor_ports = connection_properties.get('ports')
-        except KeyError:
-            msg = _("Connect volume failed, malformed connection properties")
-            raise exception.BrickException(msg=msg)
-
-        args = ['--id', user]
-        if monitor_ips and monitor_ports:
-            monitors = ["%s:%s" % (ip, port) for ip, port in
-                        zip(
-                            cls._sanitize_mon_hosts(monitor_ips),
-                            monitor_ports)]
-            for monitor in monitors:
-                args += ['--mon_host', monitor]
-
-        if conf:
-            args += ['--conf', conf]
-
-        return args
-
     @staticmethod
-    def get_rbd_device_name(pool, volume):
+    def get_rbd_device_name(pool: str, volume: str) -> str:
         """Return device name which will be generated by RBD kernel module.
 
         :param pool: RBD pool name.
@@ -179,7 +163,9 @@ class RBDConnector(base.BaseLinuxConnector):
         return '/dev/rbd/{pool}/{volume}'.format(pool=pool, volume=volume)
 
     @classmethod
-    def create_non_openstack_config(cls, connection_properties):
+    def create_non_openstack_config(
+            cls,
+            connection_properties: dict[str, Any]):
         """Get root owned Ceph's .conf file for non OpenStack usage."""
         # If keyring info is missing then we are in OpenStack, nothing to do
         keyring = connection_properties.get('keyring')
@@ -202,7 +188,10 @@ class RBDConnector(base.BaseLinuxConnector):
                                                  keyring)
         return conf
 
-    def _local_attach_volume(self, connection_properties):
+    def _local_attach_volume(
+            self,
+            connection_properties: dict[str, Any]) -> \
+            dict[str, Union[str, linuxrbd.RBDVolumeIOWrapper]]:
         # NOTE(e0ne): sanity check if ceph-common is installed.
         try:
             self._execute('which', 'rbd')
@@ -252,6 +241,7 @@ class RBDConnector(base.BaseLinuxConnector):
                 if conf:
                     rbd_privsep.delete_if_exists(conf)
 
+        res: dict[str, Union[str, linuxrbd.RBDVolumeIOWrapper]]
         res = {'path': rbd_dev_path,
                'type': 'block'}
         if conf:
@@ -259,7 +249,10 @@ class RBDConnector(base.BaseLinuxConnector):
         return res
 
     @utils.trace
-    def connect_volume(self, connection_properties):
+    @utils.connect_volume_prepare_result
+    def connect_volume(self,
+                       connection_properties: dict[str, Any]) -> \
+            dict[str, Union[linuxrbd.RBDVolumeIOWrapper, str]]:
         """Connect to a volume.
 
         :param connection_properties: The dictionary that describes all
@@ -276,7 +269,9 @@ class RBDConnector(base.BaseLinuxConnector):
         rbd_handle = self._get_rbd_handle(connection_properties)
         return {'path': rbd_handle}
 
-    def _find_root_device(self, connection_properties, conf):
+    def _find_root_device(self,
+                          connection_properties: dict[str, Any],
+                          conf) -> Optional[str]:
         """Find the underlying /dev/rbd* device for a mapping.
 
         Use the showmapped command to list all acive mappings and find the
@@ -337,8 +332,12 @@ class RBDConnector(base.BaseLinuxConnector):
         return None
 
     @utils.trace
-    def disconnect_volume(self, connection_properties, device_info,
-                          force=False, ignore_errors=False):
+    @utils.connect_volume_undo_prepare_result(unlink_after=True)
+    def disconnect_volume(self,
+                          connection_properties: dict[str, Any],
+                          device_info: dict,
+                          force: bool = False,
+                          ignore_errors: bool = False) -> None:
         """Disconnect a volume.
 
         :param connection_properties: The dictionary that describes all
@@ -370,7 +369,7 @@ class RBDConnector(base.BaseLinuxConnector):
                     rbd_handle.close()
 
     @staticmethod
-    def _check_valid_device(rbd_handle):
+    def _check_valid_device(rbd_handle: 'io.BufferedReader') -> bool:
         original_offset = rbd_handle.tell()
 
         try:
@@ -384,7 +383,9 @@ class RBDConnector(base.BaseLinuxConnector):
 
         return True
 
-    def check_valid_device(self, path, run_as_root=True):
+    def check_valid_device(self,
+                           path: Optional[str],
+                           run_as_root: bool = True) -> bool:
         """Verify an existing RBD handle is connected and valid."""
         if not path:
             return False
@@ -400,7 +401,8 @@ class RBDConnector(base.BaseLinuxConnector):
         # For backward compatibility ignore run_as_root param with handles
         return self._check_valid_device(path)
 
-    def extend_volume(self, connection_properties):
+    @utils.connect_volume_undo_prepare_result
+    def extend_volume(self, connection_properties: dict[str, Any]) -> int:
         """Refresh local volume view and return current size in bytes."""
         # Nothing to do, RBD attached volumes are automatically refreshed, but
         # we need to return the new size for compatibility
